from fanstatic import Library, Resource
from fanstatic import get_needed

foo = Library("foo", "foo_dir")

a = Resource(foo, "a.js")

b = Resource(foo, "b.js", depends=[a])

class TestSingle(object):
    def widget(self):
        a.need()
        return "the widget HTML itself"

class TestMultiple(object):
    def widget(self):
        b.need()
        return "the widget HTML itself"

class TestBottom(object):
    def widget(self):
        b.need()
        # XXX this does not use any official API and needs to be
        # reconsidered. Its done anyway now to make the tests pass,
        # instead of just removing the corresponding test.
        get_needed()._bottom = True
        get_needed()._force_bottom = True
        return "the widget HTML itself"

class TestInlineResource(object):
    pass




from zope.interface import Interface
from zope import component
from zope.publisher.interfaces.browser import IBrowserRequest

import fanstatic

from zope.fanstatic.zopesupport import ZopeFanstaticResource

def create_factory(library):
    def factory(request):
        return ZopeFanstaticResource(request, library)
    return factory

def action_setup(_context):
    """Publish all fanstatic library entry points as resources.
    """
    for library in fanstatic.get_library_registry().values():
        factory = create_factory(library)
        adapts = (IBrowserRequest,)
        provides = Interface
        _context.action(
            discriminator = ('adapter', adapts, provides, library.name),
            callable = component.provideAdapter,
            args = (factory, adapts, provides, library.name))




import unittest
import doctest

from zope.interface import Interface
from zope.component import getGlobalSiteManager
from zope.publisher.interfaces.browser import IBrowserRequest

import fanstatic
from zope.fanstatic.zcml import create_factory
from zope.fanstatic.tests.view import foo
from zope.fanstatic.testing import ZopeFanstaticBrowserLayer
import zope.fanstatic.tests

class TestLayer(ZopeFanstaticBrowserLayer):

    def testSetUp(self):
        super(TestLayer, self).testSetUp()
        # Because it is difficult to dynamically register a
        # entry_point in tests, we do the setup by hand:
        resource_factory = create_factory(foo)
        getGlobalSiteManager().registerAdapter(
            resource_factory, (IBrowserRequest,), Interface, foo.name)

    def setup_middleware(self, app):
        return fanstatic.Injector(app)

layer = TestLayer(zope.fanstatic.tests)

def test_suite():
    readme = doctest.DocFileSuite(
        '../README.txt',
        optionflags=doctest.NORMALIZE_WHITESPACE | doctest.ELLIPSIS)
    readme.layer = layer
    return unittest.TestSuite([readme])



from zope.interface import Interface

class ISetupZopeFanstatic(Interface):
    pass

class IZopeFanstaticResource(Interface):

    def get(name, default):
        pass

    def __getitem__(self, name):
        pass




from zope.interface import implements
from zope.component import adapter
from zope.publisher.interfaces import IEndRequestEvent
from zope.traversing.browser.absoluteurl import absoluteURL
from zope.traversing.browser.interfaces import IAbsoluteURL
from zope.traversing.interfaces import ITraversable

import fanstatic

from zope.fanstatic.interfaces import IZopeFanstaticResource

@adapter(IEndRequestEvent)
def set_base_url_on_needed_inclusions(event):
    # At first sight it might be better to subscribe to the
    # IBeforeTraverseEvent for ISite objects and only set a base_url
    # then. However, we might be too early in that case and miss out
    # on essential information for computing URLs. One example of such
    # information is that of the virtualhost namespace traversal.
    needed = fanstatic.get_needed()
    if not needed.has_resources():
        # Do nothing if there're no resources needed at all.
        return
    if needed.base_url is None:
        # Only set the base_url if it has not been set just yet.
        #
        # Note that the given context is set to None, resulting in
        # computing a URL to the Application root (while still
        # adhering to the, for example, virtualhost information). In
        # principle this is not correct, as we should compute the URL
        # for the nearest ISite object, but there is no site nor
        # context anymore in the EndRequestEvent (as the request has
        # been "closed", transactions have been handled, and the site
        # is cleared). Since fanstatic resource "registrations" cannot
        # be overridden on a per ISite basis anyway, this is good
        # enough.
        needed.base_url = absoluteURL(None, event.request)

_sentinel = object()

class ZopeFanstaticResource(object):

    # Hack to get ++resource++foo/bar/baz.jpg *paths* working in Zope
    # Pagetemplates. Note that ++resource+foo/bar/baz.jpg *URLs* will
    # not work with this hack!
    #
    # The ZopeFanstaticResource class also implements an __getitem__()
    # / get() interface, to support rendering URLs to resources from
    # code.

    implements(IZopeFanstaticResource, ITraversable, IAbsoluteURL)

    def __init__(self, request, library, name=''):
        self.request = request
        self.library = library
        self.name = name

    def get(self, name, default=_sentinel):
        # XXX return default if given, or NotFound (or something) when
        # not, in case name is not resolved to an actual resource.
        name = '%s/%s' % (self.name, name)
        return ZopeFanstaticResource(self.request, self.library, name=name)

    def traverse(self, name, furtherPath):
        return self.get(name)

    def __getitem__(self, name):
        resource = self.get(name, None)
        if resource is None:
            raise KeyErro(name)
        return resource

    def __str__(self):
        needed = fanstatic.get_needed()
        if needed.base_url is None:
            needed.base_url = absoluteURL(None, self.request)
        return needed.library_url(self.library) + self.name

    __call__ = __str__



import unittest
from zope.interface import Interface
from zope.component import getMultiAdapter
from zope.publisher.browser import TestRequest
from zope.traversing.interfaces import ITraversable

import fanstatic
from zope.fanstatic.zopesupport import ZopeFanstaticResource
from zope.fanstatic.tests import tests

class ComputeURL(unittest.TestCase):

    layer = tests.layer

    def setUp(self):
        fanstatic.init_needed()
        self.context = object()
        self.request = TestRequest()
        self.resource_namespace  = getMultiAdapter(
            (self.context, self.request), ITraversable, name='resource')

    def test_lookup_resource(self):
        # There's a resource library registered for the name 'foo'.
        resource = self.resource_namespace.traverse('foo', [])
        self.assert_(isinstance(resource, ZopeFanstaticResource))

    def test_library_url(self):
        resource = self.resource_namespace.traverse('foo', [])
        self.assertEquals('http://127.0.0.1/fanstatic/foo', str(resource))

    def test_get(self):
        resource = self.resource_namespace.traverse('foo', [])
        a_js = resource.get('a.js')
        self.assert_(isinstance(a_js, ZopeFanstaticResource))
        self.assertEquals('http://127.0.0.1/fanstatic/foo/a.js', str(a_js))

        woekie = resource.get('bar').get('baz').get('woekie.png')
        self.assert_(isinstance(woekie, ZopeFanstaticResource))
        self.assertEquals(
            'http://127.0.0.1/fanstatic/foo/bar/baz/woekie.png', str(woekie))

    def test_getitem(self):
        resource = self.resource_namespace.traverse('foo', [])
        a_js = resource['a.js']
        self.assert_(isinstance(a_js, ZopeFanstaticResource))
        self.assertEquals('http://127.0.0.1/fanstatic/foo/a.js', str(a_js))

        woekie = resource['bar']['baz']['woekie.png']
        self.assert_(isinstance(woekie, ZopeFanstaticResource))
        self.assertEquals(
            'http://127.0.0.1/fanstatic/foo/bar/baz/woekie.png', str(woekie))

    def test_call(self):
        resource = self.resource_namespace.traverse('foo', [])
        a_js = resource.get('a.js')
        self.assertEquals(str(a_js), a_js())




from setuptools import setup, find_packages
import os

def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

long_description = (
    read('src', 'zope', 'fanstatic', 'README.txt')
    + '\n' +
    read('CHANGES.txt')
    + '\n' +
    'Download\n'
    '********\n'
    )

setup(
    name='zope.fanstatic',
    version='0.9b',
    description="Fanstatic integration for Zope.",
    long_description=long_description,
    classifiers=['Framework :: Zope3'],
    keywords='',
    author='Zope Foundation and Contributors',
    author_email='zope-dev@zope.org',
    url='http://pypi.python.org/pypi/zope.fanstatic',
    license='ZPL 2.1',
    packages=find_packages('src'),
    namespace_packages=['zope'],
    package_dir={'': 'src'},
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'setuptools',
        'fanstatic',
        'zope.component',
        'zope.interface',
        'zope.publisher',
        'zope.traversing',
        ],
    extras_require = {
      'test': [
         'zope.annotation',
         'zope.app.publication',
         'zope.app.wsgi >= 3.10.0',
         'zope.browserpage',
         'zope.container',
         'zope.principalregistry',
         'zope.securitypolicy',
         'zope.security',
         'zope.site',
         'zope.app.appsetup',
         ],
      },
    )



from zope.app.wsgi.testlayer import BrowserLayer
import fanstatic

class ZopeFanstaticBrowserLayer(BrowserLayer):
    """ A zope testlayer with fanstatic Injector. """

    def setup_middleware(self, app):
        return fanstatic.Injector(app)

